/*************************************************************
*
* Description:
*	Example program  for pC/OS & pC/SFS
*
* Author[s]:
*	Frank Goetze
**************************************************************/

#include "../pC_SFS/inc/pC_SFS.h"
#include "../pC_SFS/inc/SFS_string.h"

#define OS_SYSTEM_TICKS_DIV   (OS_TIMER_TICKS_PER_SEC / OS_SYSTEM_TICKS_PER_SEC)

#define TASK_STK_SIZE         512

#ifdef _OS_SAMD21_
 #define SAMD21_TAU        0
 #define SAMD21_SPARKFUN   1
 #define SAMD21_15x15      2

 #define SAMD21_TYPE       SAMD21_TAU

 #define PORTA         0                       // user-LED definition
 #define LED_GROUP     PORTA
 #if (SAMD21_TYPE == SAMD21_TAU)               // -- TAU board
  #define LED_PORT     PORT_PA27
  #define LED_PIN      PIN_PA27
 #endif
 #if (SAMD21_TYPE == SAMD21_SPARKFUN)          // -- Sparkfun
  #define LED_PORT     PORT_PA17
  #define LED_PIN      PIN_PA17
 #endif
 #if (SAMD21_TYPE == SAMD21_15x15)             // -- SAMD15x15
  #define LED_PORT     PORT_PA09
  #define LED_PIN      PIN_PA09
 #endif

 void SysTick_init(void)
 {
    SYSCTRL->XOSC32K.reg  = SYSCTRL_XOSC32K_ENABLE;
    SYSCTRL->XOSC32K.reg |= SYSCTRL_XOSC32K_STARTUP(5) | SYSCTRL_XOSC32K_AAMPEN | SYSCTRL_XOSC32K_EN32K | SYSCTRL_XOSC32K_XTALEN | SYSCTRL_XOSC32K_ENABLE;
    while((SYSCTRL->PCLKSR.reg & SYSCTRL_PCLKSR_XOSC32KRDY) == 0) __NOP();
    GCLK->GENDIV.reg  = GCLK_GENDIV_DIV(1) | GCLK_GENDIV_ID(1);
    GCLK->GENCTRL.reg = GCLK_GENCTRL_ID(1) | GCLK_GENCTRL_SRC_XOSC32K | GCLK_GENCTRL_GENEN;
    GCLK->CLKCTRL.reg = GCLK_CLKCTRL_GEN_GCLK1 | GCLK_CLKCTRL_CLKEN | GCLK_CLKCTRL_ID(0) ;
    SYSCTRL->DFLLCTRL.reg  = SYSCTRL_DFLLCTRL_ENABLE;
    SYSCTRL->DFLLMUL.reg   = SYSCTRL_DFLLMUL_CSTEP(5) | SYSCTRL_DFLLMUL_FSTEP(10); 	
    SYSCTRL->DFLLMUL.reg  |= SYSCTRL_DFLLMUL_MUL(1465); // 1465 * 32768 is approx 48MHZ
    SYSCTRL->DFLLCTRL.reg |= SYSCTRL_DFLLCTRL_MODE;
    while((SYSCTRL->PCLKSR.reg & SYSCTRL_PCLKSR_DFLLLCKC) == 0) __NOP();
    NVMCTRL->CTRLB.reg |= NVMCTRL_CTRLB_RWS(1);
    GCLK->GENDIV.reg  = GCLK_GENDIV_DIV(1) | GCLK_GENDIV_ID(0);
    GCLK->GENCTRL.reg = GCLK_GENCTRL_GENEN | GCLK_GENCTRL_SRC_DFLL48M | GCLK_GENCTRL_ID(0);
    GCLK->CLKCTRL.reg = GCLK_CLKCTRL_CLKEN | GCLK_CLKCTRL_GEN_GCLK0 | GCLK_CLKCTRL_ID_DFLL48;
    if (SysTick_Config(SystemCoreClock / OS_TIMER_TICKS_PER_SEC)) {
        while (1) __NOP();
    }
 }

 void LED_init(void)
 {
    PORT->Group[LED_GROUP].DIRSET.reg = LED_PORT;
    PORT->Group[LED_GROUP].OUTSET.reg = LED_PORT;
 }

 void LED_toggle(void)
 {
    PORT->Group[LED_GROUP].OUTTGL.reg = LED_PORT;
 }
#endif

#ifdef _OS_STM32F1xx_
 #include "stm32f10x_rcc.h"
 #include "stm32f10x_gpio.h"

 #define   GCLK_CLKCTRL_ID_DFLL48_Val   0x0ul
 #define   GCLK_CLKCTRL_ID_DFLL48      (GCLK_CLKCTRL_ID_DFLL48_Val    << GCLK_CLKCTRL_ID_Pos)

 void SysTick_init(void)
 {
    SystemCoreClockUpdate();
    if (SysTick_Config(SystemCoreClock / OS_TIMER_TICKS_PER_SEC)) {
        while (1) __NOP();
    }
 }

 void LED_init(void)
 {
    GPIO_InitTypeDef GPIO_InitStructure;                   // user-LED PA1 (HAOYU HY-TinySTM103T)

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    GPIO_Write(GPIOA, GPIO_Pin_1);
    GPIO_Write(GPIOA, (U16)~GPIO_ReadOutputData(GPIOA));   // switch ON (control)
 }

 void LED_toggle(void)
 {
    GPIO_Write(GPIOA, (U16)~GPIO_ReadOutputData(GPIOA));
 }
#endif
 
#ifdef _OS_STM32F4xx_
 #include "stm32f4xx_rcc.h"
 #include "stm32f4xx_gpio.h"

 void SysTick_init(void)
 {
    SystemCoreClockUpdate();
    if (SysTick_Config(SystemCoreClock / OS_TIMER_TICKS_PER_SEC)) {
        while (1) __NOP();
    }
 }

 void LED_init(void)
 {
    GPIO_InitTypeDef GPIO_InitStructure;

    RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOF, ENABLE);  // state LED PF6 (HAOYU HY-STM32F4xxCore144)
    GPIO_InitStructure.GPIO_Pin   = GPIO_Pin_6;
    GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_OUT;
    GPIO_Init(GPIOF, &GPIO_InitStructure);
    GPIO_ToggleBits(GPIOF, GPIO_Pin_6);                    // switch ON (control)
 }

 void LED_toggle(void)
 {
    GPIO_ToggleBits(GPIOF, GPIO_Pin_6);
 }
#endif

#ifdef _OS_FE310_
 #define RTC_FREQ 32768
 void SysTick_init(void)
 {
    clear_csr(mie, MIP_MEIP);           // extern PLIC
    clear_csr(mie, MIP_MTIP);           // Timer
    clear_csr(mie, MIP_MSIP);           // Software
    CLINT->mtimecmp = CLINT->mtime + (RTC_FREQ / OS_TIMER_TICKS_PER_SEC);
    set_csr(mie, MIP_MTIP);             // Timer
    set_csr(mstatus, MSTATUS_MIE);      // General
 }

 void SysTick_update(void)
 {
    clear_csr(mie, MIP_MTIP);           // Timer
    CLINT->mtimecmp = CLINT->mtime + (RTC_FREQ / OS_TIMER_TICKS_PER_SEC);
    set_csr(mie, MIP_MTIP);             // Timer
 }

 void LED_init(void)
 {
    GPIO0->iof_en    &= ~GPIO_pin(19);
    GPIO0->out_xor   &= ~GPIO_pin(19);
    GPIO0->output_en |= GPIO_pin(19);
 }

 void LED_toggle(void)
 {
    if(GPIO0->port & GPIO_pin(19))
        GPIO0->port &= ~GPIO_pin(19);
    else
        GPIO0->port |= GPIO_pin(19);
 }

 #define F_CPU 320000000                 // mpu PLL-clock to use (320MHz)
 extern void trap_entry(void);
 void SystemInit(void)
 {
    clear_csr(mie, MIP_MEIP);
    write_csr(mtvec, &trap_entry);

 #ifdef F_CPU	// PLL
    RTC->lfrosccfg  &= ~RTC_lfrosccfg_en;
    PRCI->hfrosccfg  = PRCI_hfrosccfg_hfroscdiv_s(4) | PRCI_hfrosccfg_hfrosctrim_s(16) | PRCI_hfrosccfg_hfroscen;
    while((PRCI->hfrosccfg & PRCI_hfrosccfg_hfroscrdy) == 0);
    PRCI->pllcfg    &= ~PRCI_pllcfg_pllsel;
    SPI0->sckdiv     = 8;               // Slow down the QSPI
    PRCI->plloutdiv  = PRCI_plloutdiv_plloutdivby1 | PRCI_plloutdiv_plloutdiv_s(0);
    PRCI->pllcfg     = PRCI_pllcfg_pllrefsel | PRCI_pllcfg_pllbypass | PRCI_pllcfg_pllr_s(1) | PRCI_pllcfg_pllf_s(PLL_F_VAL) | PRCI_pllcfg_pllq_s(1);
    PRCI->pllcfg    &= ~PRCI_pllcfg_pllbypass;
    uint64_t now = CLINT->mtime;
    while (CLINT->mtime - now < 4);     // wait 100us (4 ticks of RTC)
    while((PRCI->pllcfg & PRCI_pllcfg_plllock) == 0);
    PRCI->pllcfg    |= PRCI_pllcfg_pllsel;
 #endif
 }
#endif

#ifdef _OS_AT91SAM3xxx_
 extern uint32_t SystemFrequency;

 void SysTick_init(void)
 {
    SysTick_Config(SystemFrequency/OS_TIMER_TICKS_PER_SEC);// Generate interrupt each ~10 ms
 }

 void LED_init(void)
 {
 }

 void LED_toggle(void)
 {
 }
#endif

U32 OS_SystemTicksCnt;

OS_STK_TYPE Stk1[TASK_STK_SIZE];
OS_STK_TYPE Stk2[TASK_STK_SIZE];

SFS_USER SFS_T1, SFS_T2;

void  Task1(void *Tdata) KC51_REENTRANT;
void  Task2(void *Tdata) KC51_REENTRANT;

int main(void)
{
    // ---- HW specific ----
    SystemInit();
    // ---------------------

    OS_Init();
#if OS_STK_CHECK_EN
    OS_TaskCreate(Task1, NULL, (void *)&Stk1[TASK_STK_SIZE], (void *)&Stk1, 2);
    OS_TaskCreate(Task2, NULL, (void *)&Stk2[TASK_STK_SIZE], (void *)&Stk2, 3);
#else
    OS_TaskCreate(Task1, NULL, (void *)&Stk1[TASK_STK_SIZE], 2);
    OS_TaskCreate(Task2, NULL, (void *)&Stk2[TASK_STK_SIZE], 3);
#endif

    OS_Start();
    while(1);
}

void OSTickISR(void)                                       // typical Cortex-Mx pC/OS ISR
{
#ifndef _OS_FE310_
    OS_IntEnter();
#else
    // ---- HW specific ----
    SysTick_update();
    // ---------------------
#endif
#if OS_TMR_EN
    OSTimerTick();                                         // call pC/OS-function (timers)
#endif
    if((++OS_SystemTicksCnt) == OS_SYSTEM_TICKS_DIV) {
        OS_SystemTicksCnt = 0;
        OSSystemTick();                                    // call pC/OS-function (delays / wait)
    }
#ifndef _OS_FE310_
    OS_IntExit();
#endif
}

void AppTickInit(void)
{                                                          // Configure the system tick
    OS_SystemTicksCnt = 0;
    
    // ---- HW specific ----
    SysTick_init();
    // ---------------------
}

U08 Fill[]="______________________________________________";
U08 Text[]="SFS_Test_File R/W";

void Task1(void *Tdata) KC51_REENTRANT
{
    U08            returnOk, err;
    U08            Readed[20];
    U08 OS_HUGE   *OS_Rev;
    SFS_HANDLE     fhandle = 0;
    SFS_LONG       written;
    SFS_ATTR       a;
    SFS_GET        fentry;

    Tdata = Tdata;

    AppTickInit();

    // ---- HW specific ----
    LED_init();
    // ---------------------

    OS_Rev = OS_GetRev();
    OS_Rev = OS_Rev;
    
    SFS_Init();                        // init SerialFileSystem

    returnOk = SFS_Format();
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_BecomeUser(&SFS_T1);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_ChangeDir("/");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateDir("Dir1");
    if(returnOk==SFS_NO_ERR) {
        returnOk=SFS_CreateFile("Dir1/File11", SFS_ATTR_RW, 100);
        returnOk=SFS_CreateFile("Dir1/File12", SFS_ATTR_RW, 200);
        returnOk=SFS_CreateFile("Dir1/File13", SFS_ATTR_RW, 300);
        returnOk=SFS_CreateFile("Dir1/File14", SFS_ATTR_RW, 400);
        returnOk=SFS_CreateFile("Dir1/File15", SFS_ATTR_RW, 500);
        returnOk=SFS_CreateFile("Dir1/File16", SFS_ATTR_RW, 600);
        returnOk=SFS_CreateFile("Dir1/File17", SFS_ATTR_RW, 700);
        returnOk=SFS_CreateFile("Dir1/File18", SFS_ATTR_RW, 800);
        returnOk=SFS_CreateFile("Dir1/File19", SFS_ATTR_RW, 900);
        returnOk=SFS_CreateFile("Dir1/File20", SFS_ATTR_RW, 1000);
        returnOk=SFS_CreateFile("Dir1/File21", SFS_ATTR_RW, 1100);
        returnOk=SFS_CreateFile("Dir1/File22", SFS_ATTR_RW, 1200);
        returnOk=SFS_CreateFile("Dir1/File23", SFS_ATTR_RW, 1300);
        returnOk=SFS_CreateFile("Dir1/File24", SFS_ATTR_RW, 1400);
        returnOk=SFS_CreateFile("Dir1/File25", SFS_ATTR_RW, 1500);
        err     =SFS_CreateFile("Dir1/File26", SFS_ATTR_RW, 1600);
        if(err!=SFS_NO_ERR)
            err =SFS_CreateFile("Dir1/File27", SFS_ATTR_RW, 1700);
    }
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateFile("File1", SFS_ATTR_RO | SFS_ATTR_RW, 100);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateDir("/Dir2");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateFile("Dir2/File21", SFS_ATTR_RO, 10);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateFile("/Dir2/File22", SFS_ATTR_RW, 300);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_ChangeDir("Dir2");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateDir("Dir22");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateDir("../Dir2/Dir21");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_ChangeDir("Dir21");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateFile("/Dir2/Dir21/File211", SFS_ATTR_RO, 200);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateFile("/Dir2/Dir21/RW_test", SFS_ATTR_RO, 50);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateFile("/Dir2/Dir21/RW_test2", SFS_ATTR_RW, 400);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateFile("/Dir2/Dir21/File212", SFS_ATTR_RW, 150);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_ChangeDir("..");
    if(returnOk==SFS_NO_ERR) {
        returnOk=SFS_RemoveFile("/Dir1/File11");
        returnOk=SFS_RemoveFile("/Dir1/File13");
        returnOk=SFS_RemoveFile("/Dir1/File15");
        returnOk=SFS_RemoveFile("/Dir1/File17");
        returnOk=SFS_RemoveFile("/Dir1/File19");
        returnOk=SFS_RemoveFile("/Dir1/File21");
        returnOk=SFS_RemoveFile("/Dir1/File23");
        returnOk=SFS_RemoveFile("/Dir1/File25");
        err     =SFS_RemoveFile("/Dir1/File27");
    }
    
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateDir("Dir21/Dir211");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateDir("Dir21/Dir211/Dir2111");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateDir("Dir21/Dir211/Dir2111/Dir21111");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateFile("Dir21/Dir211/Dir2111/File21111", SFS_ATTR_RW, 100);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateFile("Dir21/Dir211/Dir2111/Dir21111/File21111", SFS_ATTR_RW, 200);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateLink("Dir21/Dir211/Dir2111", "/Dir1/Dir11");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateLink("Dir21/Dir211/Dir2111/File21111", "/Dir1/File21111");
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateLink("Dir21/Dir211/Dir2111/Dir21111/File21111", "/File21111");

    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_GetEntry("Dir21/Dir211/Dir2111", &fentry);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_GetEntry("Dir21/Dir211/Dir2111/File21111", &fentry);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_GetEntry("/Dir1/File21111", &fentry);
    
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_ChangeDirTemp("/Dir1/Dir11/Dir21111");
    if(returnOk==SFS_NO_ERR) {
        fhandle=SFS_OpenFile("/Dir1/File21111", SFS_ACC_RW);
        if(!(fhandle))
            returnOk=SFS_GetErrNo();
        returnOk=SFS_CloseFile(fhandle);
    }
    if(returnOk==SFS_NO_ERR) {
        fhandle=SFS_OpenFile("File21111", SFS_ACC_RW);
        if(!(fhandle))
            returnOk=SFS_GetErrNo();
        returnOk=SFS_CloseFile(fhandle);
    }
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_BackDirTemp();
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_RemoveLink("/Dir1/File21111");
    if(returnOk==SFS_NO_ERR)
        err=SFS_RemoveFile("/File21111");
    if(returnOk==SFS_NO_ERR)
        err=SFS_RemoveDir("/Dir1/Dir11");
    
    OS_TimeDly(5);
    if(returnOk==SFS_NO_ERR)
        returnOk=SFS_CreateFile("../File2", SFS_ATTR_RW, 500);
    if(returnOk==SFS_NO_ERR) {
        returnOk=SFS_AttribFile("Dir21/RW_test", SFS_ATTR_RW);
        if (returnOk==SFS_NO_ERR) {
            a = SFS_GetFileAttrib("/Dir2/Dir21/RW_test");
            if ((a & SFS_ATTR_RW) != SFS_ATTR_RW)
                returnOk=SFS_GetErrNo();
            fhandle=SFS_OpenFile("/Dir2/Dir21/RW_test", SFS_ACC_RW);
            if(!(fhandle))
                returnOk=SFS_GetErrNo();
        }
        if(returnOk==SFS_NO_ERR) {
            written=SFS_WriteFile(Fill , sizeof(Fill), fhandle);
            if(written != sizeof(Fill))
                returnOk=SFS_GetErrNo();
            if(returnOk==SFS_NO_ERR)
                returnOk=SFS_SeekFile(45, fhandle);
            if(returnOk==SFS_NO_ERR) {
                written=SFS_WriteFile(Text , sizeof(Text), fhandle);
                if(written != sizeof(Text))
                    returnOk=SFS_GetErrNo();
                if(returnOk==SFS_NO_ERR) {
                    returnOk=SFS_SeekFile(45, fhandle);
                    if(returnOk==SFS_NO_ERR) {
                        written=SFS_ReadFile((U08 OS_HUGE *)Readed, sizeof(Text), fhandle);
                        if(written != sizeof(Text))
                            returnOk=SFS_GetErrNo();
                    }
                }
            }
            returnOk=SFS_CloseFile(fhandle);
            if(returnOk==SFS_NO_ERR)
                returnOk=SFS_RenameFile("/Dir2/Dir21/RW_test","config");
        }
    }

    while(1) {
        fhandle = SFS_OpenFile("/Dir2/Dir21/config", SFS_ACC_RW);
        if(fhandle) {
            returnOk = SFS_SeekFile(45, fhandle);
            if(returnOk == SFS_NO_ERR) {
                written = SFS_WriteFile(Text , sizeof(Text), fhandle);
                if(written != sizeof(Text))
                    returnOk = SFS_GetErrNo();
                if(returnOk == SFS_NO_ERR) {
                    returnOk = SFS_SeekFile(45, fhandle);
                    if(returnOk == SFS_NO_ERR) {
                        written = SFS_ReadFile((U08 OS_HUGE *)Readed, sizeof(Text), fhandle);
                        if(written != sizeof(Text))
                            returnOk=SFS_GetErrNo();
                        else if(0 != SFSstrcmp(Text, Readed))
                            while(1);
                    }
                }
            }
            returnOk = SFS_CloseFile(fhandle);
            // ---- HW specific ----
            LED_toggle();
            // ---------------------
        } else {
            returnOk = SFS_GetErrNo();
            if(returnOk == SFS_NO_ERR) 
                while(1);
        }
        OS_TimeDly(1);
    }
//    returnOk = SFS_CloseUser();
}

void Task2(void *Tdata) KC51_REENTRANT
{
    U08            returnOk;
    U08            Readed[20];
    SFS_HANDLE     fhandle;
    SFS_LONG       written, loop = 0;

    Tdata = Tdata;

    returnOk = OS_TimeDlyResume(2);
    returnOk = SFS_BecomeUser(&SFS_T2);
    if(returnOk == SFS_NO_ERR) {
        while(1) {
            fhandle=SFS_OpenFile("/Dir2/Dir21/config", SFS_ACC_RW);
            if(fhandle) {
                returnOk = SFS_SeekFile(45, fhandle);
                if(returnOk == SFS_NO_ERR) {
                    written = SFS_WriteFile(Text , sizeof(Text), fhandle);
                    if(written != sizeof(Text))
                        returnOk = SFS_GetErrNo();
                    if(returnOk == SFS_NO_ERR) {
                        returnOk = SFS_SeekFile(45, fhandle);
                        if(returnOk == SFS_NO_ERR) {
                            written = SFS_ReadFile((U08 OS_HUGE *)Readed, sizeof(Text), fhandle);
                            if(written != sizeof(Text))
                                returnOk = SFS_GetErrNo();
                            else if(0 != SFSstrcmp(Text, Readed))
                                while(1);
                        }
                    }
                }
                returnOk = SFS_CloseFile(fhandle);
            // ---- HW specific ----
            LED_toggle();
            // ---------------------
            } else {
                returnOk = SFS_GetErrNo();
                if(returnOk == SFS_NO_ERR) 
                    while(1);
            }
            loop++;
            OS_TimeDly(1);
        }
//        returnOk = SFS_CloseUser();
    }
}



