/***********************************************************
*                       pC/SFS 1.01b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                      SFS_LLdriver.h                      *
*                      lowlevel driver                     *
*                    compiler-spezifics                    *
***********************************************************/
#ifndef _SFS_LL_
 #define _SFS_LL_

 #include "pC_SFS.h"

/*
***********************************************************
*                    LLdriver SPI LOCK
***********************************************************
*/
#ifdef _pC_OS_
 #define sfsPort_lock()    OS_Lock()
 #define sfsPort_unlock()  OS_Unlock()
#else
 #define sfsPort_lock()                // add your interrupt or sheduling lock functions here
 #define sfsPort_unlock()
#endif

/*
***********************************************************
*                    SFS PORT PROTOTYPES
***********************************************************
*/
U08 SFSPort_Init(void) KC51_REENTRANT;
U08 SFSPort_Read(U32 offs, U08 OS_HUGE *dest, SFS_LONG length) KC51_REENTRANT;
U08 SFSPort_Write(U32 offs, U08 OS_HUGE *source, SFS_LONG length) KC51_REENTRANT;
U08 SFSPort_Fill(U32 offs, U08 value, SFS_LONG length) KC51_REENTRANT;

/*
************************************************************
*                    SFS host hardware PORT
************************************************************
* for the SPI/I2C interface we have to differ on the host
* platform...
* but we only differ for this LLdriver in the C-file
************************************************************
*/
#include "../inc/LLdriver_HW/SFS_Port_SPI_HWxxxx.h"        // any SPI HW to xRAM
#include "../inc/LLdriver_HW/SFS_Port_I2C_HWxxxx.h"        // any I2C HW to xRAM

/************************* END ****************************/
#endif
