/***********************************************************
*                       pC/SFS 1.01b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
*                  SFS_Port_SPI_ARMv8M.c                   *
*                ARMv8M FPGA SPI xRAM PORT                 *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../../inc/LLdriver_HW/SFS_Port_SPI_HWxxxx.h"

#include "SMM_MPS2.h"

/***********************************************************
 *   CortexM23 FPGA configuration
 ***********************************************************
 * Connector   Pin         Signal      Alternate Func    Description
 * -----------------------------------------------------------------
 * EXP1        Pin25       EXP11       SPI3 SCK          Shield 0: SPI Clock
 * EXP1        Pin31       EXP12       SPI3 SS           Shield 0: SPI Chip Select
 * EXP1        Pin33       EXP13       SPI3 MOSI         Shield 0: SPI Data Out
 * EXP1        Pin2        EXP14       SPI3 MISO         Shield 0: SPI Data In
 **********************************************************/

#define SSP3_nCS     (1UL<<12)
#define SSP_CS(x)   ((x) ? (IOTKIT_SECURE_GPIO0->DATA |= SSP3_nCS) : (IOTKIT_SECURE_GPIO0->DATA &= ~SSP3_nCS))
#define SSP_TFE       0x01
#define SSP_RNE       0x04
#define SSP_BSY       0x10

/*
************************************************************
*             LOWLEVEL SET /CS TO ACTIVE
************************************************************
*/
void _sfsPort_CS_en(void)
{
#ifdef _pC_OS_
    OS_Lock();
#endif
    SSP_CS(0);
}
/*
************************************************************
*             LOWLEVEL RESET /CS TO DEACTIVE
************************************************************
*/
void _sfsPort_CS_dis(void)
{
    SSP_CS(1);
#ifdef _pC_OS_
    OS_Unlock();
#endif
}

/*
************************************************************
*             LOWLEVEL BYTE_WISE I/O-FUNCTION
************************************************************
*/
U08 _sfsPort_RW(U08 byte)
{
    while (!(MPS2_SECURE_SSP3->SR & SSP_TFE)) __NOP();
    MPS2_SECURE_SSP3->DR = byte;
    while (!(MPS2_SECURE_SSP3->SR & SSP_RNE)) __NOP();
    return (U08)MPS2_SECURE_SSP3->DR;
}

/*
************************************************************
*             LOWLEVEL READ STATUS
************************************************************
*/
U08 _sfsPort_readStatus(U08 cmd)
{
    U08 sr;

    _sfsPort_CS_en();
    _sfsPort_RW(cmd);
    sr = _sfsPort_RW(0xFF);
    _sfsPort_CS_dis();
    return sr;
}
/*
************************************************************
*                 INIT SPI-INTERFACE
************************************************************
*/
void _sfsPort_Init(void)
{
    /* Configure SSP3 used for SPI MRAM                                         */
    MPS2_SECURE_SSP3->CR1   =   0;                  /* Synchronous serial port disable  */
    MPS2_SECURE_SSP3->DMACR =   0;                  /* Disable FIFO DMA                 */
    MPS2_SECURE_SSP3->IMSC  =   0;                  /* Mask all FIFO/IRQ interrupts     */
    MPS2_SECURE_SSP3->ICR   =  ((1ul <<  0) |       /* Clear SSPRORINTR interrupt       */
                                (1ul <<  1) );      /* Clear SSPRTINTR interrupt        */
    MPS2_SECURE_SSP3->CR0   =  ((7ul <<  0) |       /* 8 bit data size                  */
                                (0ul <<  4) |       /* Motorola frame format            */
                                (0ul <<  6) |       /* CPOL = 0                         */
                                (0ul <<  7) |       /* CPHA = 0                         */
                                (1ul <<  8) );      /* Set serial clock rate            */
    MPS2_SECURE_SSP3->CPSR  =   (2ul <<  0);        /* set SSP clk to 6MHz (6.6MHz max) */
    MPS2_SECURE_SSP3->CR1   =  ((1ul <<  1) |       /* Synchronous serial port enable   */
                                (0ul <<  2) );      /* Device configured as master      */
    IOTKIT_SECURE_GPIO0->ALTFUNCSET |= ( (1<<14) | (1<<13) | (1<<11) );
    // manual use of /SS line of SSP3
    IOTKIT_SECURE_GPIO0->OUTENSET = SSP3_nCS;
    IOTKIT_SECURE_GPIO0->DATA    |= SSP3_nCS;
/*
    while(1) {
        _NOP(); _NOP(); _NOP();
        _sfsPort_CS_en();
        _sfsPort_RW(0x55);
        _sfsPort_CS_dis();
        _NOP(); _NOP(); _NOP();
    }
*/
}

/************************* END ****************************/
