/***********************************************************
*                       pC/SFS 1.01b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                SFS_LLdriver_xRAM_Win32.c                 *
*                lowlevel driver Host image                *
*             (to split lowlevel & HW specific)            *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/
#include "iostream"
#include <windows.h>
#include <tchar.h>
#include <stdio.h>
#include <strsafe.h>


extern "C" {

#include "../inc/pC_SFS.h"

#if !defined(SFS_MEM_xRAM_Port)
 #error "wrong config for the RAM image SFS-port!"
#endif

#define IMAGE_FILE TEXT("./SFS_IMAGE.bin")

/**********************************************************
 *   using an RAM array as memory with image-support
 *   
 *   SFS_MEMORY_START [ SFS_MEMORY_SIZE ]
 *   
 **********************************************************/
extern U08 SFS_MEMORY_START[SFS_MEMORY_SIZE];

/*
************************************************************
*                LOAD DRIVE from a real File
************************************************************
*/
S16 SFSPort_Load()
{
    HANDLE        ifile;
    S32           fsize;
    DWORD         bytesread = 0;

    ifile = CreateFile(IMAGE_FILE, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    if(ifile == INVALID_HANDLE_VALUE)
        return -1;
    SetFilePointer(ifile, 0, NULL, FILE_BEGIN);
    fsize = GetFileSize(ifile, NULL);
    if(fsize != SFS_MEMORY_SIZE) {
        CloseHandle(ifile);
        return 1;
    } else {
        SetFilePointer(ifile, 0, NULL, FILE_BEGIN);
        ReadFile(ifile, SFS_MEMORY_START, SFS_MEMORY_SIZE, &bytesread, NULL);
        CloseHandle(ifile);
    }
    return SFS_NO_ERR;
}

/*
************************************************************
*                  FLUSH DRIVE to a real File
************************************************************
*/
S16 SFS_Flush()
{
    HANDLE       ofile;
    DWORD        byteswritten = 0;

    ofile = CreateFile(IMAGE_FILE, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
    if(ofile == INVALID_HANDLE_VALUE)
        return -1;
    SetFilePointer(ofile, 0, NULL, FILE_BEGIN);
    WriteFile(ofile, SFS_MEMORY_START, SFS_MEMORY_SIZE, &byteswritten, NULL);
    CloseHandle(ofile);
    return(SFS_NO_ERR);
}

/**********************************************************/
}     // extern "C"


/************************* END ****************************/
