/***********************************************************
*                       pC/SFS 1.01b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
* Permission to use, copy, modify, and distribute this     *
* software in source and binary forms and its              *
* documentation for any purpose and without fee is hereby  *
* granted, provided that the above authors notice appear   *
* in all copies and that both that authors notice and this *
* permission notice appear in supporting documentation.    *
*                                                          *
* THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND   *
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT    *
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
* IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,  *
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
* USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
* OF SUCH DAMAGE.                                          *
************************************************************
*                        SFS_hash.c                        *
*                   string-hash-functions                  *
*                 (to not use any library)                 *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../inc/pC_SFS.h"

/*
************************************************************
*                    LENGHT OF A STRING                    *
************************************************************
*/
SFS_LONG SFSstrlen(U08 OS_HUGE *string) KC51_REENTRANT
{
    SFS_LONG byte;

    byte = 0;
    while((string[byte] != 0x00) && (byte < (SFS_LONG)(0-1))) byte++;
    return(byte);
}

/*
************************************************************
*                POS OF A CHAR IN A STRING                 *
************************************************************
*/
SFS_LONG SFSstrpos(U08 OS_HUGE *string, U08 c) KC51_REENTRANT
{
    SFS_LONG byte;

    byte = 0;
    while((string[byte] != c) && (string[byte] != 0x00) && (byte < (SFS_LONG)(0-1))) byte++;
    if(string[byte] != c) {
        return((SFS_LONG)(0-1));
    }
    return(byte);
}

/*
************************************************************
*           POS OF A CHAR IN A STRING FROM RIGHT           *
************************************************************
*/
SFS_LONG SFSstrrpos(U08 OS_HUGE *string, U08 c) KC51_REENTRANT
{
    SFS_LONG byte;

    byte = SFSstrlen(string);
    if(byte > 0) {
        while((string[byte] != c) && (byte > 0)) byte--;
    }
    if((byte == 0) && (string[byte] != c)) {
        return((SFS_LONG)(0-1));
    }
    return(byte);
}

/*
************************************************************
*                   COMPARE TWO STRINGS                    *
************************************************************
*/
U08 SFSstrcmp(U08 OS_HUGE *string1, U08 OS_HUGE *string2) KC51_REENTRANT
{
    SFS_LONG byte;

    byte = 0;
    while((string1[byte] == string2[byte]) && (string1[byte] != 0x00)) byte++;
    if(string1[byte] != string2[byte]) {
        return((U08)(0-1));
    }
    return(0);
}

/*
************************************************************
*                COMPARE n BYTE OF TWO STRINGS             *
************************************************************
*/
U08 SFSstrncmp(U08 OS_HUGE *string1, U08 OS_HUGE *string2, SFS_LONG bytes) KC51_REENTRANT
{
    SFS_LONG byte;

    byte = 0;
    while((string1[byte] == string2[byte]) && (string1[byte] != 0x00) && (byte < bytes)) byte++;
    if(byte < bytes) {
        return((U08)(0-1));
    }
    return(0);
}

/************************* END ****************************/

