/***********************************************************
*                       pC/SFS 1.01b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
*                 SFS_Port_I2C_STM32F1xx.c                 *
*                 STM32F1xx I2C xRAM PORT                  *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../../inc/LLdriver_HW/SFS_Port_I2C_HWxxxx.h"

#include "stm32f10x_rcc.h"
#include "stm32f10x_gpio.h"
#include "stm32f10x_i2c.h"

/***********************************************************
 *   STMicro STM32F1xx configuration
 ***********************************************************
 *
 *   PB7 --- I2C1 ----------------------- SDA     ->    (NO internal pull-up possible !)
 *   PB6 --- I2C1 ----------------------- SCL     ->    (NO internal pull-up possible !)
 *
 *   !!! external pull-up's are required (2.2k) !!!
 **********************************************************/

/*
************************************************************
*                LOWLEVEL READ BYTES
************************************************************
*/
U08 _sfsPort_readBytes(U32 addr, U08 *data, SFS_LONG len)
{
    SFS_LONG i;

    while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY));
    I2C_GenerateSTART(I2C1, ENABLE);
    while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT));
    I2C_Send7bitAddress(I2C1, I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK, I2C_Direction_Transmitter);
    while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));
    for(i = 2; i > 0; i--) {
        I2C_SendData(I2C1,(addr >> (8*(i-1))) & 0xFF);
        while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
    }
    I2C_GenerateSTART(I2C1, ENABLE);
    while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT));
    I2C_Send7bitAddress(I2C1, I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK, I2C_Direction_Receiver);
    I2C_AcknowledgeConfig(I2C1, ENABLE);
    for(i = 0; i < (len-1); i++) {
        while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED));
        data[i] = I2C_ReceiveData(I2C1);
    }
    I2C_AcknowledgeConfig(I2C1, DISABLE);
    if(len) {
        while (!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED));
        data[len-1] = I2C_ReceiveData(I2C1);
    }
    I2C_GenerateSTOP(I2C1, ENABLE);
    while(I2C_GetFlagStatus(I2C1, I2C_FLAG_STOPF));
    return SFS_NO_ERR;
}

/*
************************************************************
*                LOWLEVEL WRITE BYTES
************************************************************
*/
U08 _sfsPort_writeBytes(U32 addr, U08 *data, SFS_LONG len)
{
    SFS_LONG i;

    while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY));
    I2C_GenerateSTART(I2C1, ENABLE);
    while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT));
    I2C_Send7bitAddress(I2C1, I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK, I2C_Direction_Transmitter);
    while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));
    for(i = 2; i > 0; i--) {
        I2C_SendData(I2C1,(addr >> (8*(i-1))) & 0xFF);
        while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
    }
    for(i = 0; i < len; i++) {
        I2C_SendData(I2C1, data[i]);
        while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
    }
    I2C_GenerateSTOP(I2C1, ENABLE);
    while(I2C_GetFlagStatus(I2C1, I2C_FLAG_STOPF));
    return SFS_NO_ERR;
}

/*
************************************************************
*                LOWLEVEL FILL BYTES
************************************************************
*/
U08 _sfsPort_fillBytes(U32 addr, U08 value, SFS_LONG len)
{
    SFS_LONG i;

    while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY));
    I2C_GenerateSTART(I2C1, ENABLE);
    while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT));
    I2C_Send7bitAddress(I2C1, I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK, I2C_Direction_Transmitter);
    while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED));
    for(i = 2; i > 0; i--) {
        I2C_SendData(I2C1,(addr >> (8*(i-1))) & 0xFF);
        while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
    }
    for(i = 0; i < len; i++) {
        I2C_SendData(I2C1, value);
        while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED));
    }
    I2C_GenerateSTOP(I2C1, ENABLE);
    while(I2C_GetFlagStatus(I2C1, I2C_FLAG_STOPF));
    return SFS_NO_ERR;
}

/*
************************************************************
*                LOWLEVEL WAIT READY
************************************************************
*/
void _sfsPort_writeReady(void)
{
    U32 s, addr = 0;

    while(I2C_GetFlagStatus(I2C1, I2C_FLAG_BUSY));
    do {
        I2C_GenerateSTART(I2C1, ENABLE);
        while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT));
        I2C_Send7bitAddress(I2C1, I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK, I2C_Direction_Transmitter);
        do { s = I2C1->SR1; } while((s & (I2C_SR1_ADDR | I2C_SR1_AF)) == 0);
        if(s & I2C_SR1_ADDR) {
            (void)I2C1->SR2;
            I2C_GenerateSTOP(I2C1, ENABLE);
            while(I2C_GetFlagStatus(I2C1, I2C_FLAG_STOPF));
            return;
        } else
            I2C_ClearFlag(I2C1, I2C_FLAG_AF);                  
    } while(1);
}

/*
************************************************************
*                 INIT I2C-INTERFACE
************************************************************
*/
void _sfsPort_Init(void)
{
    GPIO_InitTypeDef  GPIO_InitStructure;
    I2C_InitTypeDef   I2C_InitStructure;

    RCC_APB1PeriphClockCmd(RCC_APB1Periph_I2C1,ENABLE);
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB| RCC_APB2Periph_AFIO , ENABLE);
    GPIO_InitStructure.GPIO_Pin   =  GPIO_Pin_6 | GPIO_Pin_7;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF_OD;
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    I2C_DeInit(I2C1);
    I2C_InitStructure.I2C_Mode        = I2C_Mode_I2C;
    I2C_InitStructure.I2C_DutyCycle   = I2C_DutyCycle_16_9;
    I2C_InitStructure.I2C_OwnAddress1 = 1;
    I2C_InitStructure.I2C_Ack         = I2C_Ack_Enable;
    I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
    I2C_InitStructure.I2C_ClockSpeed  = 100000;
    I2C_Cmd(I2C1, ENABLE);
    I2C_Init(I2C1, &I2C_InitStructure);
    I2C_AcknowledgeConfig(I2C1, ENABLE);
}

/************************* END ****************************/
